/// <reference types="react" />
import * as React from "react";
declare module "react" {
    function createContext<Store>(): Context<Store>;
}
export interface IProviderProps<Store> {
    value: Store;
    children?: React.ReactNode;
}
export interface IConsumerProps<Store> {
    children: {
        (store: Store): React.ReactNode;
    };
}
export declare type Provider<Store> = React.ComponentClass<IProviderProps<Store>>;
export declare type Consumer<Store> = React.ComponentClass<IConsumerProps<Store>>;
export interface Context<Store> {
    Provider: Provider<Store>;
    Consumer: Consumer<Store>;
}
export declare type AnyComponent<P> = React.ComponentClass<P> | React.StatelessComponent<P>;
export declare type Modifier<State> = (state: State) => State;
export declare type Middleware<State> = (modifier: Modifier<State>) => Modifier<State>;
export declare type Dispatch<State> = (modifier: Modifier<State>) => State;
export declare type MapStateToProps<State, OwnProps, PropsFromState> = (state: State, ownProps: OwnProps) => PropsFromState;
export declare type MapDispatchToProps<State, PropsWithDispatch> = (dispatch: Dispatch<State>) => PropsWithDispatch;
export declare type Connect<State> = <OwnProps, PropsFromState, PropsWithDispatch>(mapStateToProps: MapStateToProps<State, OwnProps, PropsFromState>, mapDispatchToProps: MapDispatchToProps<State, PropsWithDispatch>) => (componentToWrap: AnyComponent<PropsFromState & PropsWithDispatch>) => React.StatelessComponent<OwnProps>;
export interface Store<State> {
    getState: () => State;
    dispatch: Dispatch<State>;
}
export interface IBazaarConsumerProps<State> {
    children: (store: Store<State>) => React.ReactNode;
}
export interface IBazaarContext<State> {
    Provider: React.ComponentClass;
    Consumer: React.StatelessComponent<IBazaarConsumerProps<State>>;
    connect: Connect<State>;
}
export declare function createBazaar<State>(initialState: State, middleware?: Array<Middleware<State>>): IBazaarContext<State>;
