'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var React = require('react');

/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */
/* global Reflect, Promise */

var extendStatics = Object.setPrototypeOf ||
    ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
    function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };

function __extends(d, b) {
    extendStatics(d, b);
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
}

var __assign = Object.assign || function __assign(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
    }
    return t;
};

function createBazaar(initialState, middleware) {
    if (middleware === void 0) { middleware = []; }
    var baseContext = React.createContext();
    var Provider = baseContext.Provider, Consumer = baseContext.Consumer;
    var BazaarProvider = /** @class */ (function (_super) {
        __extends(BazaarProvider, _super);
        function BazaarProvider(props) {
            var _this = _super.call(this, props) || this;
            _this.bazaarStateHistory = [initialState];
            _this.getCurrentState = function () { return _this.bazaarStateHistory[_this.bazaarStateHistory.length - 1]; };
            _this.state = initialState;
            _this.dispatch = function (modifier) {
                var next = modifier;
                middleware.slice().reverse().forEach(function (middleware) {
                    next = middleware(next);
                });
                var newState = next(_this.getCurrentState());
                _this.bazaarStateHistory.push(newState);
                _this.forceUpdate();
                return newState;
            };
            return _this;
        }
        BazaarProvider.prototype.render = function () {
            var value = {
                dispatch: this.dispatch,
                getState: this.getCurrentState
            };
            return (React.createElement(Provider, { value: value }, this.props.children));
        };
        return BazaarProvider;
    }(React.Component));
    function connect(mapStateToProps, mapDispatchToProps) {
        if (mapDispatchToProps === void 0) { mapDispatchToProps = null; }
        return function (ComponentToWrap) {
            var Purify = /** @class */ (function (_super) {
                __extends(Purify, _super);
                function Purify() {
                    return _super !== null && _super.apply(this, arguments) || this;
                }
                Purify.prototype.shouldComponentUpdate = function (nextProps) {
                    var _this = this;
                    var should = false;
                    Object.keys(nextProps.propsFromState).forEach(function (key) {
                        var currValues = _this.props.propsFromState;
                        var nextValues = nextProps.propsFromState;
                        if (currValues[key] != nextValues[key])
                            should = true;
                    });
                    return should;
                };
                Purify.prototype.render = function () {
                    var Comp = this.props.component;
                    return React.createElement(Comp, __assign({}, this.props.propsFromState, this.props.propsWithDispatch), this.props.children);
                };
                return Purify;
            }(React.Component));
            var WrappedComponent = function (ownProps) { return (React.createElement(Consumer, null, function (store) { return React.createElement(Purify, { propsFromState: mapStateToProps(store.getState(), ownProps), propsWithDispatch: mapDispatchToProps == null ? null : mapDispatchToProps(store.dispatch), component: ComponentToWrap }, ownProps.children); })); };
            return WrappedComponent;
        };
    }
    var BazaarConsumer = function (props) { return (React.createElement(Consumer, null, function (store) { return props.children(store); })); };
    return {
        Consumer: BazaarConsumer,
        Provider: BazaarProvider,
        connect: connect
    };
}

exports.createBazaar = createBazaar;
