import * as React from "react"
import { createBazaar, Modifier } from "../bazaar"

interface ITodo {
	id: number,
	description: string,
	done: boolean
}

interface IState {
    todos: Array<ITodo>
}

const initialState : IState = {
    todos: [
		{ id: 1, description: "Learn Typescript", done: true },
		{ id: 2, description: "Use React-Bazaar", done: false }
	]
}

const context = createBazaar(initialState)

// a dumb Todo-Component
interface ITodoComponentProps { todo: ITodo }
const Todo = (props: ITodoComponentProps) =>
	<div style={{textDecoration: props.todo.done ? "line-through" : "none"}}>
		{props.todo.description}
	</div>

// a list of dumb Todo-Components, rendered from the todos in the store
const Todos = () => (
    <context.Consumer>
        {store => store.getState().todos.map(todo => <Todo todo={todo} />)}
    </context.Consumer>
)


const toggleTodo = (todoId: number) : Modifier<IState> => store =>
{
	const todoToToggle = store.todos.find(todo => todo.id === todoId)
	if(todoToToggle)
		todoToToggle.done = !todoToToggle.done
	return store
}

// if you want replayability of your state you can use an immutability helper library like immer
import immer from "immer"
const toggleTodo2 = (todoId: number) : Modifier<IState> => state => immer(state, state =>
{
	const todoToToggle = state.todos.find(todo => todo.id === todoId)
	if(todoToToggle)
		todoToToggle.done = !todoToToggle.done
})

interface ITodoComponentProps2 { todo: ITodo, toggle: { () : void } }
const Todo2 = (props: ITodoComponentProps2) =>
	<div
		onClick={props.toggle}
		style={{textDecoration: props.todo.done ? "line-through" : "none"}}>
		{props.todo.description}
	</div>

const Todos2 = () => (
    <context.Consumer>
		{
			store =>
				store.getState().todos.map(todo => <Todo2 todo={todo} toggle={() => store.dispatch(toggleTodo(todo.id))}/>)
		}
    </context.Consumer>
)
